/* Script: rwl_apihelper.js
Version 1
Autor: Rene Friedrich

Description:
helper with ajax wrapper to do simple webapi requests
for CRUD (create, read, update, delete) operations

Sample: Update a Row
updateTableRow(
    "rwl_verguetungsverhandlungs", 
   "aee2d262-b654-ef11-bfe3-000d3a450a2a", 
     {
        "rwl_name": "VVH-NASE",
        "rwl_description": "New Description",
        "rwl_status": "Active"
    }, 
    function(res) {
        console.log("Successfully updated:", res);
    }
); 
--- End updata row sample

Sample: Select Data from a row

var result;
selectTableRow(
    "rwl_verguetungsverhandlungs", 
    "aee2d262-b654-ef11-bfe3-000d3a450a2a", 
    ["rwl_name", "rwl_description", "rwl_status"]
)
.then(data => {
    console.log("Retrieved data:", data);
    result = data;
})
.catch(error => {
    console.error("Error:", error);
});

--- End select data sample
*/

/* ----- CODE START ----- */

// AJAX Wrapper
(function(webapi, $){
    function safeAjax(ajaxOptions) {
        var deferredAjax = $.Deferred();

        shell.getTokenDeferred().done(function (token) {
            // add headers for AJAX
            if (!ajaxOptions.headers) {
                $.extend(ajaxOptions, {
                    headers: {
                        "__RequestVerificationToken": token
                    }
                }); 
            } else {
                ajaxOptions.headers["__RequestVerificationToken"] = token;
            }
            $.ajax(ajaxOptions)
                .done(function(data, textStatus, jqXHR) {
                    validateLoginSession(data, textStatus, jqXHR, deferredAjax.resolve);
                }).fail(deferredAjax.reject); //AJAX
        }).fail(function () {
            deferredAjax.rejectWith(this, arguments); // on token failure pass the token AJAX and args
        });

        return deferredAjax.promise();	
    }
    webapi.safeAjax = safeAjax;
})(window.webapi = window.webapi || {}, jQuery)

// Update Table Row with a given data object
function updateTableRow(tablename, recordId, dataObj, successCallback) {
    const url = `/_api/${tablename}(${recordId})`;

    webapi.safeAjax({
        type: "PATCH",
        url: url,
        contentType: "application/json",
        data: JSON.stringify(dataObj),  // Convert the data object to a JSON string
        success: function (res) {
            if (typeof successCallback === 'function') {
                successCallback(res);
            } else {
                console.log("Update successful:", res);
            }
        },
        error: function (xhr, status, error) {
            console.error("Error during update:", status, error);
        }
    });
}

// select data from a table
function selectTableRow(tablename, recordId, fieldset) {
    // Dynamically create the URL for the GET request
    const url = `/_api/${tablename}(${recordId})?$select=${fieldset.join(",")}`;

    // Return a promise
    return new Promise((resolve, reject) => {
        // Perform the AJAX request
        webapi.safeAjax({
            type: "GET",
            url: url,
            contentType: "application/json",
            success: function (res) {
                // Resolve the promise with the retrieved data
                resolve(res);
            },
            error: function (xhr, status, error) {
                // Reject the promise with an error
                reject(new Error(`Error during data retrieval: ${status} ${error}`));
            }
        });
    });
}

/* Example usage of the function with promise handling
var result;
selectTableRow(
    "rwl_verguetungsverhandlungs", 
    "aee2d262-b654-ef11-bfe3-000d3a450a2a", 
    ["rwl_name", "rwl_description", "rwl_status"]
)
.then(data => {
    console.log("Retrieved data:", data);
    result = data;
})
.catch(error => {
    console.error("Error:", error);
});
*/